package edu.nyu.scps.tree;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Stack;

import junit.framework.Assert;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.content.DialogInterface;
import android.content.res.Resources;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Toast;

public class TreeActivity extends ListActivity {
	final int DIALOG_CREATE_ID = 0;
	EditText editText;
	int selectedItem;	//item in ListView when context menu appeared

	//relation of new item with respect to selectedItem
	enum Relation {
		ABOVE,	//new item will go above selectedItem
		BELOW,	//new item will go below selectedItem
		CHILD,	//new item will be 1st child of selectedItem
		PARENT	//new item will be parent of selectedItem
	}
	Relation relation;
	
	private class Item {
		public String name;
		public ArrayList<Item> subitems;
		
		public Item(String n, ArrayList<Item> i) {
			name = n;
			subitems = i;
		}
	}
	
	Item root = new Item("Root", new ArrayList<Item>(Arrays.asList(new Item[] {
		new Item("America", new ArrayList<Item>(Arrays.asList(new Item[] {
			new Item("New York", new ArrayList<Item>(Arrays.asList(new Item[] {
				new Item("Rensselaer", new ArrayList<Item>(Arrays.asList(new Item[] {
					new Item("Rensselaer", new ArrayList<Item>(Arrays.asList(new Item[] {
					}))),
					new Item("Kinderhook", new ArrayList<Item>(Arrays.asList(new Item[] {
					})))
				}))),
				new Item("Columbia", new ArrayList<Item>(Arrays.asList(new Item[] {
					new Item("Hudson", new ArrayList<Item>(Arrays.asList(new Item[] {
					})))
				}))),
				new Item("Dutchess", new ArrayList<Item>(Arrays.asList(new Item[] {
					new Item("Hyde Park", new ArrayList<Item>(Arrays.asList(new Item[] {
					}))),
					new Item("Poughkeepsie", new ArrayList<Item>(Arrays.asList(new Item[] {
					}))),
					new Item("Wappingers Falls", new ArrayList<Item>(Arrays.asList(new Item[] {
					}))),
					new Item("New Hamburg", new ArrayList<Item>(Arrays.asList(new Item[] {
					}))),
					new Item("Beacon", new ArrayList<Item>(Arrays.asList(new Item[] {
					})))
				}))),
				new Item("Putnam", new ArrayList<Item>(Arrays.asList(new Item[] {
					new Item("Cold Spring", new ArrayList<Item>(Arrays.asList(new Item[] {
					}))),
					new Item("Garrison", new ArrayList<Item>(Arrays.asList(new Item[] {
					})))
				}))),
				new Item("Westchester", new ArrayList<Item>(Arrays.asList(new Item[] {
					new Item("Peekskill", new ArrayList<Item>(Arrays.asList(new Item[] {
					}))),
					new Item("Croton", new ArrayList<Item>(Arrays.asList(new Item[] {
					}))),
					new Item("Ossining", new ArrayList<Item>(Arrays.asList(new Item[] {
					}))),
					new Item("Tarrytown", new ArrayList<Item>(Arrays.asList(new Item[] {
					}))),
					new Item("Irvington", new ArrayList<Item>(Arrays.asList(new Item[] {
					}))),
					new Item("Dobbs Ferry", new ArrayList<Item>(Arrays.asList(new Item[] {
					}))),
					new Item("Hastings", new ArrayList<Item>(Arrays.asList(new Item[] {
					}))),
					new Item("Yonkers", new ArrayList<Item>(Arrays.asList(new Item[] {
					})))
				})))
			}))),

			new Item("New Jersey", new ArrayList<Item>(Arrays.asList(new Item[] {
				new Item("Bergen", new ArrayList<Item>(Arrays.asList(new Item[] {
					new Item("Fort Lee", new ArrayList<Item>(Arrays.asList(new Item[] {
					}))),
					new Item("Englewood", new ArrayList<Item>(Arrays.asList(new Item[] {
					}))),
					new Item("Alpine", new ArrayList<Item>(Arrays.asList(new Item[] {
					})))
				})))
			}))),

			new Item("Connecticut", new ArrayList<Item>(Arrays.asList(new Item[] {
				new Item("Fairfield", new ArrayList<Item>(Arrays.asList(new Item[] {
					new Item("Cos Cob", new ArrayList<Item>(Arrays.asList(new Item[] {
					})))
				})))
			})))
		})))
	})));

	//used only when the Android back button is pressed.
	Stack<Item> stack = new Stack<Item>();
	
	//stack.peek() is the item at the top of the stack.
	//Display this item.
	
	public void update() {
		Resources resources = getResources(); 
		CharSequence appName =
			resources.getText(resources.getIdentifier("app_name", "string", getPackageName()));

		ArrayList<Item> subitems = stack.peek().subitems;
		int size = subitems.size();
		String s = String.format("%s: %s (level %d) has %d subitem%s.",
			appName, stack.peek().name, stack.size() - 1, size,
			size == 1 ? "" : "s"
		);
		setTitle(s);
		
		String[] a = new String[size];
		for (int i = 0; i < size; ++i) {
			a[i] = subitems.get(i).name;
		}

		ArrayAdapter<String> adapter = new ArrayAdapter<String>(
			TreeActivity.this,
			android.R.layout.simple_list_item_1,
			a
		);
		setListAdapter(adapter);
	}

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		//setContentView(R.layout.main);
		editText = new EditText(this);

		//The initial list is the list of states within America.
		stack.push(root);
		stack.push(root.subitems.get(0));
		update();

		ListView listView = getListView();
		registerForContextMenu(listView);

		listView.setOnItemClickListener(new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				stack.push(stack.peek().subitems.get(position));
				TreeActivity.this.update();
			}
		});	
	}

	//When the user presses the Android back button,
	//go one step back towards the root of the tree.

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		if ((keyCode == KeyEvent.KEYCODE_BACK)) {
			if (stack.size() > 1) {
				stack.pop();
				TreeActivity.this.update();
			}
		}
		return true;
	}

	//When the user holds their finger down on an item,
	//display a context menu.

	@Override
	public void onCreateContextMenu(ContextMenu menu, View v,
		ContextMenuInfo menuInfo) {
		
		super.onCreateContextMenu(menu, v, menuInfo);
		selectedItem = (int)((AdapterContextMenuInfo)menuInfo).id;

		//If the item is not already the top item,
		if (selectedItem > 0) {	
			menu.add(Menu.NONE, 0, 0, "move up");
		}
		
		//If the item is not already the bottom item,
		if (selectedItem < stack.peek().subitems.size() - 1) {
			menu.add(Menu.NONE, 1, 1, "move down");
		}

		menu.add(Menu.NONE, 2, 2, "insert above");
		menu.add(Menu.NONE, 3, 3, "insert below");
		menu.add(Menu.NONE, 4, 4, "delete");

		if (stack.peek().subitems.get(selectedItem).subitems.isEmpty()) {
			menu.add(Menu.NONE, 5, 5, "create first child");
		}
		
		if (stack.size() == 1) {
			menu.add(Menu.NONE, 6, 6, "create parent");
		}
	}

	//Add a new Item.

	void add(String name) {
		Item item = new Item(name, new ArrayList<Item>());
		ArrayList<Item> subitems = stack.peek().subitems;

		switch (relation) {
		case ABOVE:
			Assert.assertTrue(selectedItem > 0);
			subitems.add(selectedItem - 1, item);
			break;

		case BELOW:
			subitems.add(selectedItem, item);
			break;

		case CHILD:
			Assert.assertTrue(subitems.get(selectedItem).subitems.isEmpty());
			subitems.get(selectedItem).subitems.add(item);
			break;
			
		case PARENT:
			Assert.assertTrue(stack.size() == 1);
			item.subitems.add(subitems.get(selectedItem));
			stack.peek().subitems.remove(subitems.get(selectedItem));
			stack.peek().subitems.add(item);
			break;

		default:
			Toast.makeText(this, "bad relation " + relation, Toast.LENGTH_SHORT).show();
		}

		update();
	}

	@Override
	protected Dialog onCreateDialog(int id, Bundle bundle) {
		Assert.assertEquals(id, DIALOG_CREATE_ID);
		AlertDialog.Builder builder =
			new AlertDialog.Builder(TreeActivity.this);
		builder.setTitle("Name for new item:");
		builder.setView(editText);

		builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int whichButton) {
				TreeActivity.this.add(editText.getText().toString());
			}
		});
		
		builder.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int whichButton) {
				TreeActivity.this.add(editText.getText().toString());
			}
		});

		return builder.create();
	}
	
	protected void onPrepareDialog (int id, Dialog dialog, Bundle args) {
		editText.setText("");
	}

	@Override
	public boolean onContextItemSelected(MenuItem menuItem) {
		//selectedItem is the zero-based index of the selected item in the ListView.
		AdapterContextMenuInfo menuInfo =
			(AdapterContextMenuInfo)menuItem.getMenuInfo();
		selectedItem = (int)menuInfo.id;

		//menuItem is the zero-based index of the selected item in the context menu.
		switch (menuItem.getItemId()) {
		case 0:	//move up
			Item i = stack.peek().subitems.remove(selectedItem);
			stack.peek().subitems.add(selectedItem - 1, i);
			break;
			
		case 1:	//move down
			Item j = stack.peek().subitems.remove(selectedItem);
			stack.peek().subitems.add(selectedItem + 1, j);
			break;
			
		case 2:	//insert above
			relation = Relation.ABOVE;
			showDialog(DIALOG_CREATE_ID);
			break;
			
		case 3:	//insert below
			relation = Relation.BELOW;
			showDialog(DIALOG_CREATE_ID);
			break;

		case 4:	//delete
			String s = "Goodbye, " + stack.peek().subitems.get(selectedItem).name + "!";
			Toast.makeText(this, s, Toast.LENGTH_SHORT).show();
			stack.peek().subitems.remove(selectedItem);
			break;
		
		case 5:	//create first child
			relation = Relation.CHILD;
			showDialog(DIALOG_CREATE_ID);
			break;
			
		case 6:	//create parent
			relation = Relation.PARENT;
			showDialog(DIALOG_CREATE_ID);
			break;

		default:
		    return super.onContextItemSelected(menuItem);
		}

		update();
		return true;
	}
}